@@ -21,12 +21,15 @@ int main(int argc, char *argv[])
     if (parse_config(argv[1]) != 0) {
         return 2;
     }
-    if (open_db_log() != 0) {
+    if (app_context.daemonize != 0 && do_daemonize() != 0) {
         return 3;
     }
-    if (http_server() != 0) {
+    if (open_db_log() < 0) {
         return 4;
     }
+    if (http_server() != 0) {
+        return 5;
+    }
     free_db_log();
     free_config();
     
