@@ -4,20 +4,11 @@
 #include "domain_records.h"
 #include "domain_layers.h"
 #include "query_parser.h"
+#include "expirables.h"
 
-#define INT_PROPERTY_COMMON_PREFIX     '_'
-#define INT_PROPERTY_EXPIRES_AT        "_expires_at"
-#define INT_PROPERTY_TYPE              "_type"
-#define INT_PROPERTY_TTL               "_ttl"
-#define INT_PROPERTY_POSITION          "_loc"
-#define INT_PROPERTY_DELETE_PREFIX     "_delete:"
-#define INT_PROPERTY_DELETE_ALL_PREFIX "_delete_all"
-#define INT_PROPERTY_ADD_INT_PREFIX    "_add_int:"
-
-#define PROPERTIES_DEFAULT_SLIP_MAP_BUFFER_SIZE (size_t) 32U
-
-RB_PROTOTYPE_STATIC(KeyNodes_, KeyNode_, entry, key_node_cmp);
-RB_GENERATE_STATIC(KeyNodes_, KeyNode_, entry, key_node_cmp);
+#ifndef PROPERTIES_DEFAULT_SLIP_MAP_BUFFER_SIZE
+# define PROPERTIES_DEFAULT_SLIP_MAP_BUFFER_SIZE (size_t) 32U
+#endif
 
 typedef struct RecordsPutOptParseCBContext_ {
     RecordsPutOp *put_op;
@@ -55,6 +46,18 @@ static int records_put_opt_parse_cb(void * const context_,
         }
         *zeroed1 = ',';
         put_op->position_set = 1;
+    } else if (BINVAL_IS_EQUAL_TO_CONST_STRING(key, INT_PROPERTY_EXPIRES_AT)) {
+        char *svalue = value->val;
+        skip_spaces((const char * *) &svalue);
+        if (*svalue == 0) {
+            put_op->expires_at = (time_t) 0;
+            return 0;
+        }
+        char *endptr;
+        put_op->expires_at = (time_t) strtoul(svalue, &endptr, 10);
+        if (endptr == NULL || endptr == svalue) {
+            return -1;
+        }
     } else {
         assert(key->size > (size_t) 0U);
         if (*key->val == INT_PROPERTY_COMMON_PREFIX) {
@@ -103,6 +106,7 @@ int handle_domain_records(struct evhttp_request * const req,
         }
         *sep = 0;
         if ((layer_name = new_key_from_c_string(uri)) == NULL) {
+            *sep = '/';
             return HTTP_SERVUNAVAIL;
         }
         *sep++ = '/';
@@ -111,8 +115,7 @@ int handle_domain_records(struct evhttp_request * const req,
             return HTTP_NOTFOUND;
         }
         if ((key = new_key_from_c_string(sep)) == NULL) {
-            release_key(layer_name);
-            
+            release_key(layer_name);            
             return HTTP_SERVUNAVAIL;
         }
         *get_op = (RecordsGetOp) {
@@ -155,8 +158,7 @@ int handle_domain_records(struct evhttp_request * const req,
             return HTTP_NOTFOUND;
         }
         if ((key = new_key_from_c_string(sep)) == NULL) {
-            release_key(layer_name);
-            
+            release_key(layer_name);            
             return HTTP_SERVUNAVAIL;
         }
         evbuffer_add(evhttp_request_get_input_buffer(req), "", (size_t) 1U);
@@ -176,7 +178,8 @@ int handle_domain_records(struct evhttp_request * const req,
             },
             .position_set = 0,
             .properties = NULL,
-            .special_properties = NULL
+            .special_properties = NULL,
+            .expires_at = (time_t) 0
         };
         RecordsPutOptParseCBContext cb_context = {
             .put_op = put_op
@@ -222,8 +225,7 @@ int handle_domain_records(struct evhttp_request * const req,
             return HTTP_NOTFOUND;
         }
         if ((key = new_key_from_c_string(sep)) == NULL) {
-            release_key(layer_name);
-            
+            release_key(layer_name);            
             return HTTP_SERVUNAVAIL;
         }
         *delete_op = (RecordsDeleteOp) {
@@ -403,9 +405,21 @@ int handle_op_records_put(RecordsPutOp * const put_op,
     }
     assert(status >= 0);
     if (status == 0 && put_op->position_set != 0 && key_node->slot != NULL) {
-        remove_entry_from_key_node(pan_db, key_node, 0);
-        assert(key_node->slot != NULL);
-        key_node->slot = NULL;
+#if PROJECTION
+        const Position2D * const previous_position =
+            &key_node->slot->real_position;
+#else
+        const Position2D * const previous_position =
+            &key_node->slot->position;
+#endif
+        if (previous_position->latitude == put_op->position.latitude &&
+            previous_position->longitude == put_op->position.longitude) {
+            put_op->position_set = 0;
+        } else {
+            remove_entry_from_key_node(pan_db, key_node, 0);
+            assert(key_node->slot != NULL);
+            key_node->slot = NULL;
+        }
     }
     if (put_op->position_set != 0) {
         Slot slot;
@@ -422,7 +436,7 @@ int handle_op_records_put(RecordsPutOp * const put_op,
         if (add_slot(pan_db, &slot, &new_slot) != 0) {
             RB_REMOVE(KeyNodes_, &pan_db->key_nodes, key_node);
             key_node->slot = NULL;
-            free_key_node(key_node);
+            free_key_node(pan_db, key_node);
             free_slip_map(&put_op->properties);
             free_slip_map(&put_op->special_properties);            
             
@@ -450,6 +464,31 @@ int handle_op_records_put(RecordsPutOp * const put_op,
                          &cb_context);
         free_slip_map(&put_op->properties);
     }
+    Expirable *expirable = key_node->expirable;
+    if (put_op->expires_at != (time_t) 0) {
+        if (expirable == NULL) {
+            Expirable new_expirable = {
+                .ts = put_op->expires_at,
+                .key_node = key_node
+            };
+            expirable = add_entry_to_slab(&context->expirables_slab,
+                                          &new_expirable);
+            key_node->expirable = expirable;
+            add_expirable_to_tree(pan_db, expirable);
+        } else {
+            if (expirable->ts != put_op->expires_at) {
+                remove_expirable_from_tree(pan_db, expirable);
+                expirable->ts = put_op->expires_at;
+                add_expirable_to_tree(pan_db, expirable);
+            }
+        }
+        assert(expirable->key_node == key_node);
+    } else if (expirable != NULL) {
+        assert(expirable->key_node == key_node);
+        remove_expirable_from_tree(pan_db, expirable);        
+        remove_entry_from_slab(&context->expirables_slab, expirable);
+        key_node->expirable = NULL;
+    }
     if (put_op->fake_req != 0) {
         return 0;
     }
@@ -564,7 +603,7 @@ int handle_op_records_delete(RecordsDeleteOp * const delete_op,
     }
     assert(key_node->slot == NULL);
     RB_REMOVE(KeyNodes_, &pan_db->key_nodes, key_node);
-    free_key_node(key_node);
+    free_key_node(pan_db, key_node);
     if (delete_op->fake_req != 0) {
         return 0;
     }
