@@ -2,6 +2,7 @@
 #include "common.h"
 #include "http_server.h"
 #include "query_parser.h"
+#include "domain_records.h"
 #include "domain_system.h"
 
 static int handle_special_op_system_rewrite(HttpHandlerContext * const context,
@@ -35,6 +36,10 @@ int handle_domain_system(struct evhttp_request * const req,
         return 0;
     }
     if (req->type == EVHTTP_REQ_POST && strcasecmp(uri, "shutdown") == 0) {
+        if (app_context.db_log.journal_rewrite_process != (pid_t) -1) {
+            kill(app_context.db_log.journal_rewrite_process, SIGKILL);
+            app_context.db_log.journal_rewrite_process = (pid_t) -1;
+        }
         event_base_loopbreak(context->event_base);
         return 0;
     }
@@ -111,6 +116,7 @@ char *get_tmp_log_file_name(void)
 typedef struct RebuildJournalLayerCBContext_ {
     HttpHandlerContext *context;
     int tmp_log_fd;
+    _Bool ts_written;
 } RebuildJournalLayerCBContext;
 
 static int flush_log_buffer(struct evbuffer *log_buffer, const int log_fd)
@@ -201,13 +207,13 @@ static int rebuild_journal_record_cb(void *context_, KeyNode * const key_node)
     const int verb = EVHTTP_REQ_PUT;
     const Key * const key = key_node->key;
     const size_t uri_len =
-        context->http_handler_context->encoded_base_uri_len +
+        context->http_handler_context->encoded_api_base_uri_len +
         sizeof "records/" - (size_t) 1U + context->encoded_layer_name->size +
         sizeof "/" - (size_t) 1U + key->len - (size_t) 1U +
         sizeof ".json" - (size_t) 1U;
     evbuffer_add_printf(log_buffer, "%x %zx:%srecords/%s/%s.json ",
                         verb, uri_len,
-                        context->http_handler_context->encoded_base_uri,
+                        context->http_handler_context->encoded_api_base_uri,
                         context->encoded_layer_name->val, key->val);
 
     struct evbuffer * const body_buffer = evbuffer_new();
@@ -225,15 +231,26 @@ static int rebuild_journal_record_cb(void *context_, KeyNode * const key_node)
         if (cb_context.first == 0) {
             evbuffer_add(body_buffer, "&", (size_t) 1U);
         }
+        cb_context.first = 0;
 #if PROJECTION
     const Position2D * const position = &key_node->slot->real_position;
 #else
     const Position2D * const position = &key_node->slot->position;
 #endif
-        evbuffer_add_printf(body_buffer, "_loc=%f,%f",
+        evbuffer_add_printf(body_buffer, 
+                            INT_PROPERTY_POSITION "=%f,%f",
                             (double) position->latitude,
                             (double) position->longitude);
     }
+    if (key_node->expirable != NULL) {
+        if (cb_context.first == 0) {
+            evbuffer_add(body_buffer, "&", (size_t) 1U);
+        }
+        cb_context.first = 0;
+        evbuffer_add_printf(body_buffer,
+                            INT_PROPERTY_EXPIRES_AT "=%lu",
+                            (unsigned long) key_node->expirable->ts);
+    }
     evbuffer_add_printf(log_buffer, "%zx:", evbuffer_get_length(body_buffer));
     evbuffer_add_buffer(log_buffer, body_buffer);    
     evbuffer_free(body_buffer);
@@ -253,11 +270,19 @@ static int rebuild_journal_layer_cb(void *context_, void *entry,
     Layer * const layer = entry;
     int ret = 0;    
     assert(layer->name != NULL && *layer->name != 0);
-    printf("Dumping layer: [%s] ...\n", layer->name);
+    logfile(context->context, LOG_INFO, "Dumping layer: [%s] ...",
+            layer->name);
     struct evbuffer *log_buffer;
     if ((log_buffer = evbuffer_new()) == NULL) {
         return -1;
     }
+    if (context->ts_written == 0) {
+        const time_t ts = time(NULL);
+        if (ts != (time_t) -1 &&
+            add_ts_to_ev_log_buffer(log_buffer, ts) == 0) {
+            context->ts_written = 1;
+        }
+    }
     evbuffer_add(log_buffer, DB_LOG_RECORD_COOKIE_HEAD,
                  sizeof DB_LOG_RECORD_COOKIE_HEAD - (size_t) 1U);
     const int verb = EVHTTP_REQ_POST;
@@ -273,11 +298,11 @@ static int rebuild_journal_layer_cb(void *context_, void *entry,
         evbuffer_free(log_buffer);
         return -1;
     }
-    const size_t uri_len = context->context->encoded_base_uri_len +
+    const size_t uri_len = context->context->encoded_api_base_uri_len +
         sizeof "layers/" - (size_t) 1U + encoded_layer_name.size +
         sizeof ".json" - (size_t) 1U;
     evbuffer_add_printf(log_buffer, "%x %zx:%slayers/%s.json %zx:",
-                        verb, uri_len, context->context->encoded_base_uri,
+                        verb, uri_len, context->context->encoded_api_base_uri,
                         encoded_layer_name.val, (size_t) 0U);
     evbuffer_add(log_buffer, DB_LOG_RECORD_COOKIE_TAIL,
                  sizeof DB_LOG_RECORD_COOKIE_TAIL - (size_t) 1U);
@@ -313,7 +338,8 @@ int rebuild_journal(HttpHandlerContext * const context, const int tmp_log_fd)
 {
     RebuildJournalLayerCBContext cb_context = {
         .context = context,
-        .tmp_log_fd = tmp_log_fd
+        .tmp_log_fd = tmp_log_fd,
+        .ts_written = 0
     };
     slab_foreach(&context->layers_slab, rebuild_journal_layer_cb,
                  &cb_context);
@@ -333,7 +359,8 @@ int rewrite_child(HttpHandlerContext * const context)
         close(db_log->db_log_fd);
     }
     nice(BGREWRITEAOF_NICENESS);
-    puts("Creating a new journal as a background process...");    
+    logfile_noformat(context, LOG_INFO,
+                     "Creating a new journal as a background process...");
     char *tmp_log_file_name = get_tmp_log_file_name();
     if (tmp_log_file_name == NULL) {
         return -1;
@@ -349,12 +376,13 @@ int rewrite_child(HttpHandlerContext * const context)
     flags |= O_LARGEFILE;
 #endif
     int db_log_fd = open(tmp_log_file_name, flags, (mode_t) 0600);
-    free(tmp_log_file_name);
     if (db_log_fd == -1) {
-        fprintf(stderr, "Can't create [%s]: [%s]\n", tmp_log_file_name,
-                strerror(errno));
+        logfile(context, LOG_ERR, "Can't create [%s]: [%s]", tmp_log_file_name,
+               strerror(errno));
+        free(tmp_log_file_name);
         return -1;
     }
+    free(tmp_log_file_name);    
     rebuild_journal(context, db_log_fd);
     fsync(db_log_fd);
     close(db_log_fd);    
@@ -461,7 +489,8 @@ int system_rewrite_after_fork_cb(void)
     if (tmp_log_file_name == NULL) {
         return -1;
     }
-    puts("Completing the new journal with recent transactions...");
+    logfile_noformat(NULL, LOG_INFO,
+                     "Completing the new journal with recent transactions...");
     assert(db_log->offset_before_fork != (off_t) -1);
     if (lseek(db_log->db_log_fd, db_log->offset_before_fork,
               SEEK_SET) == (off_t) -1) {
@@ -481,7 +510,7 @@ int system_rewrite_after_fork_cb(void)
 #endif
     int tmp_log_fd = open(tmp_log_file_name, flags, (mode_t) 0600);
     if (tmp_log_fd == -1) {
-        fprintf(stderr, "Can't reopen [%s]: [%s]\n", tmp_log_file_name,
+        logfile(NULL, LOG_ERR, "Can't reopen [%s]: [%s]", tmp_log_file_name,
                 strerror(errno));
         unlink(tmp_log_file_name);
     }
@@ -497,17 +526,19 @@ int system_rewrite_after_fork_cb(void)
     printf("Renaming [%s] to [%s]\n", tmp_log_file_name,
            db_log->db_log_file_name);
     if (rename(tmp_log_file_name, db_log->db_log_file_name) != 0) {
-        perror("rename()");
+        logfile(NULL, LOG_ERR, "Unable to rename [%s] to [%s]: [%s]",
+                tmp_log_file_name, db_log->db_log_file_name,
+                strerror(errno));
         unlink(tmp_log_file_name);
         free(tmp_log_file_name);
         return -1;
     }
     free(tmp_log_file_name);
     if (close(db_log->db_log_fd) != 0) {
-        perror("Unable to close the previous journal");
+        logfile_error(NULL, "Unable to close the previous journal");
     }
     db_log->db_log_fd = tmp_log_fd;
-    puts("Done - New journal activated");
+    logfile_noformat(NULL, LOG_INFO, "Done - New journal activated");
     
     return 0;
 }
