@@ -2,11 +2,8 @@
 #include "common.h"
 #include "pandb.h"
 
-RB_PROTOTYPE_STATIC(KeyNodes_, KeyNode_, entry, key_node_cmp);
-RB_GENERATE_STATIC(KeyNodes_, KeyNode_, entry, key_node_cmp);
-
-void get_qrects_from_qbounds(Rectangle2D qrects[4],
-                             const Rectangle2D * const qbounds)
+static void get_qrects_from_qbounds(Rectangle2D qrects[4],
+                                    const Rectangle2D * const qbounds)
 {
     const Dimension median_latitude =
         (qbounds->edge0.latitude + qbounds->edge1.latitude) / 2.0;
@@ -32,8 +29,8 @@ void get_qrects_from_qbounds(Rectangle2D qrects[4],
     qrects[3].edge1.longitude = median_longitude;    
 }
 
-int position_is_in_rect(const Position2D * const position,
-                        const Rectangle2D * const rect)
+static int position_is_in_rect(const Position2D * const position,
+                               const Rectangle2D * const rect)
 {
     if (position->latitude >= rect->edge0.latitude &&
         position->longitude >= rect->edge0.longitude &&
@@ -44,11 +41,11 @@ int position_is_in_rect(const Position2D * const position,
     return 0;
 }
 
-Node *find_node_for_position(const QuadNode * const quad_node,
-                             const Rectangle2D * const qrects,
-                             const Position2D * const position,
-                             Rectangle2D * const rect_pnt,
-                             unsigned int *part_id)
+static Node *find_node_for_position(const QuadNode * const quad_node,
+                                    const Rectangle2D * const qrects,
+                                    const Position2D * const position,
+                                    Rectangle2D * const rect_pnt,
+                                    unsigned int *part_id)
 {
     unsigned int t = 4U;
     
@@ -93,7 +90,7 @@ void free_slot(Slot * const slot)
     slot->bucket_node = NULL;
 }
 
-int init_bucket(Bucket * const bucket)
+static int init_bucket(Bucket * const bucket)
 {
     init_slab(&bucket->slab, sizeof(Slot), "slots");
     bucket->bucket_size = (NbSlots) BUCKET_SIZE;
@@ -115,7 +112,7 @@ static int free_bucket_cb(void *context, void *entry,
     return 0;
 }
 
-void free_bucket(Bucket * const bucket)
+static void free_bucket(Bucket * const bucket)
 {
     if (bucket == NULL) {
         return;
@@ -126,8 +123,8 @@ void free_bucket(Bucket * const bucket)
     bucket->busy_slots = (NbSlots) 0U;    
 }
 
-int init_bucket_node(BucketNode * const bucket_node,
-                     const QuadNode * const parent)
+static int init_bucket_node(BucketNode * const bucket_node,
+                            const QuadNode * const parent)
 {
     bucket_node->type = NODE_TYPE_BUCKET_NODE;
     assert(parent->type == NODE_TYPE_QUAD_NODE);
@@ -137,7 +134,7 @@ int init_bucket_node(BucketNode * const bucket_node,
     return 0;
 }
 
-void free_bucket_node(BucketNode * const bucket_node)
+static void free_bucket_node(BucketNode * const bucket_node)
 {
     if (bucket_node == NULL) {
         return;
@@ -148,7 +145,7 @@ void free_bucket_node(BucketNode * const bucket_node)
     free(bucket_node);
 }
 
-BucketNode *new_bucket_node(const QuadNode * const parent)
+static BucketNode *new_bucket_node(const QuadNode * const parent)
 {
     BucketNode *bucket_node;
 
@@ -162,7 +159,7 @@ BucketNode *new_bucket_node(const QuadNode * const parent)
     return bucket_node;
 }
 
-int init_quad_node(QuadNode * const quad_node)
+static int init_quad_node(QuadNode * const quad_node)
 {
     quad_node->type = NODE_TYPE_QUAD_NODE;
     quad_node->parent = NULL;
@@ -175,7 +172,7 @@ int init_quad_node(QuadNode * const quad_node)
     return 0;
 }
 
-void free_quad_node(QuadNode * const quad_node)
+static void free_quad_node(QuadNode * const quad_node)
 {
     if (quad_node == NULL) {
         return;
@@ -190,7 +187,7 @@ void free_quad_node(QuadNode * const quad_node)
     free(quad_node);
 }
 
-QuadNode *new_quad_node(void)
+static QuadNode *new_quad_node(void)
 {
     QuadNode *quad_node;
     
@@ -203,9 +200,9 @@ QuadNode *new_quad_node(void)
     return quad_node;
 }
 
-int add_slot_to_bucket(PanDB * const db, BucketNode * const bucket_node,
-                       const Slot * const slot, int update_sub_slots,
-                       Slot * * const new_slot)
+static int add_slot_to_bucket(PanDB * const db, BucketNode * const bucket_node,
+                              const Slot * const slot, int update_sub_slots,
+                              Slot * * const new_slot)
 {
     QuadNode *parent;
     Bucket *bucket;
@@ -241,8 +238,8 @@ typedef struct RebalanceBucketCBContext_ {
     Rectangle2D *qrects_;
 } RebalanceBucketCBContext;
 
-int rebalance_bucket_cb(void *context_, void *entry,
-                        const size_t sizeof_entry)
+static int rebalance_bucket_cb(void *context_, void *entry,
+                               const size_t sizeof_entry)
 {
     Node * target_node;    
     Slot * scanned_slot = (Slot *) entry;
@@ -367,7 +364,7 @@ int remove_entry_from_key_node(PanDB * const db,
     if (should_free_key_node != 0) {
         RB_REMOVE(KeyNodes_, &db->key_nodes, key_node);        
         key_node->slot = NULL;
-        free_key_node(key_node);
+        free_key_node(db, key_node);
     }
     bucket_node = slot->bucket_node;
     assert(bucket_node != NULL);
@@ -466,8 +463,8 @@ int remove_entry(PanDB * const db, Key * const key)
     return 0;
 }
 
-int rectangle2d_intersect(const Rectangle2D * const r1,
-                          const Rectangle2D * const r2)
+static int rectangle2d_intersect(const Rectangle2D * const r1,
+                                 const Rectangle2D * const r2)
 {
     if (!(r1->edge0.longitude > r2->edge1.longitude ||
           r1->edge1.longitude < r2->edge0.longitude ||
@@ -487,8 +484,8 @@ typedef struct FindNearIntCBContext_ {
     void *context_cb;
 } FindNearIntCBContext;
 
-int find_near_context_cb(void *context_, void *entry,
-                         const size_t sizeof_entry)
+static int find_near_context_cb(void *context_, void *entry,
+                                const size_t sizeof_entry)
 {
     FindNearIntCBContext *context = context_;
     Slot *scanned_slot = entry;
@@ -620,9 +617,9 @@ static inline Dimension dimension_max(const Dimension d1,
     return d1 > d2 ? d1 : d2;    
 }
 
-unsigned int find_zones(const PanDB * const db,
-                        const Rectangle2D * const rect_,
-                        Rectangle2D rects[4])
+static unsigned int find_zones(const PanDB * const db,
+                               const Rectangle2D * const rect_,
+                               Rectangle2D rects[4])
 {
     const Rectangle2D qbounds = db->qbounds;
     Rectangle2D *rect_pnt = &rects[0];    
@@ -742,7 +739,7 @@ int find_near(const PanDB * const db,
         matching_rects[0] = rect;        
         nb_zones = 1U;
     } else {
-        nb_zones = find_zones(db, &rect , matching_rects);
+        nb_zones = find_zones(db, &rect, matching_rects);
     }
     assert(nb_zones >= 1U);
     assert(nb_zones <= 4U);
@@ -751,6 +748,7 @@ int find_near(const PanDB * const db,
     if (stack_inspect == NULL) {
         return -1;
     }
+    assert(matching_rect == &matching_rects[0]);
     int ret;
     do {
         ret = find_near_in_zone(matching_rect,
@@ -761,6 +759,7 @@ int find_near(const PanDB * const db,
                                 position,
                                 distance,
                                 limit);
+        matching_rect++;
     } while (ret == 0 && --nb_zones > 0U);
     free_pnt_stack(stack_inspect);
     
@@ -773,11 +772,12 @@ typedef struct FindInRectIntCBContext_ {
     const Rectangle2D *rect;
     SubSlots limit;
     FindInRectCB cb;
+    FindInRectClusterCB cluster_cb;
     void *context_cb;
 } FindInRectIntCBContext;
 
-int find_in_rect_context_cb(void *context_, void *entry,
-                            const size_t sizeof_entry)
+static int find_in_rect_context_cb(void *context_, void *entry,
+                                   const size_t sizeof_entry)
 {
     FindInRectIntCBContext *context = context_;
     Slot *scanned_slot = entry;
@@ -797,42 +797,85 @@ int find_in_rect_context_cb(void *context_, void *entry,
                                              &scanned_slot->position);
     }
     if (scanned_slot->key_node != NULL) {
+        if (context->limit <= (SubSlots) 0U) {
+            return 1;
+        }
+        context->limit--;
         if (context->cb != NULL) {
             const int ret =
                 context->cb(context->context_cb, scanned_slot, cd);
             if (ret != 0) {
                 return ret;
             }
-            if (context->limit-- <= (SubSlots) 1U) {
-                return 1;
-            }
         }
     } else {
-        printf("=== [NULL KEY] => %.3f/%.3f (%.2f)\n",
-               scanned_slot->position.latitude,
-               scanned_slot->position.longitude,
-               cd);
+        fprintf(stderr, "=== [NULL KEY] => %.3f/%.3f (%.2f)\n",
+                scanned_slot->position.latitude,
+                scanned_slot->position.longitude,
+                cd);
     }
     return 0;
 }
 
-int find_in_rect(const PanDB * const db,
-                 FindInRectCB cb, void * const context_cb,
-                 const Rectangle2D * const rect, const SubSlots limit)
+static int find_in_rect_cluster_context_cb(void *context_, void *entry,
+                                           const size_t sizeof_entry)
 {
-    if (limit <= (SubSlots) 0) {
+    FindInRectIntCBContext *context = context_;
+    if (context->cluster_cb == NULL) {
         return 0;
     }
+    QuadNode *scanned_node = entry;
+    const Rectangle2D * const rect = context->rect;
+    const Position2D center = {
+        .latitude = (rect->edge1.latitude + rect->edge0.latitude) / 2.0,
+        .longitude = (rect->edge1.longitude + rect->edge0.longitude) / 2.0
+    };
+    if (position_is_in_rect(&center, context->rect) == 0) {
+        return 0;
+    }    
+    const Dimension distance =
+        (fabsf(center.latitude - rect->edge0.latitude) +
+            fabsf(center.longitude - rect->edge0.longitude)) / 2.0;
+    Meters radius;
+    if (context->db->layer_type == LAYER_TYPE_SPHERICAL ||
+        context->db->layer_type == LAYER_TYPE_GEOIDAL) {
+        radius = geoidal_distance_to_meters(distance);
+    } else {
+        radius = (Meters) distance;
+    }
+    assert(scanned_node->type == NODE_TYPE_QUAD_NODE);
+    const NbSlots children = scanned_node->sub_slots;
+    (void) sizeof_entry;
+    
+    const int ret =
+        context->cluster_cb(context->context_cb, &center, radius, children);
+    if (ret != 0) {
+        return ret;
+    }
+    if (context->limit-- <= (SubSlots) 1U) {
+        return 1;
+    }
+    return 0;
+}
+
+static int find_in_rect_in_zone(Rectangle2D * const matching_rect,
+                                PntStack *stack_inspect,
+                                const PanDB * const db,
+                                FindInRectCB cb,
+                                FindInRectClusterCB cluster_cb,
+                                void * const context_cb,
+                                const Rectangle2D * const rect,
+                                const SubSlots limit, const Dimension epsilon)
+{
     const Position2D rect_center = {
         .latitude =
         (rect->edge1.latitude + rect->edge0.latitude) / (Dimension) 2.0,
         .longitude =
         (rect->edge1.longitude + rect->edge0.longitude) / (Dimension) 2.0
     };
+    const _Bool cluster = (epsilon > (Dimension) 0.0);
+    SubSlots max_nb_slots_without_clustering = (SubSlots) 0U;
     Rectangle2D scanned_qbounds = db->qbounds;
-    Rectangle2D matching_rects[1];
-    Rectangle2D *matching_rect = &matching_rects[0];    
-    PntStack *stack_inspect;
     const QuadNode *scanned_node;
     Rectangle2D scanned_children_qbounds[4];
     Rectangle2D *scanned_child_qbound;
@@ -840,14 +883,12 @@ int find_in_rect(const PanDB * const db,
     QuadNodeWithBounds *sqnb;
     Node *scanned_node_child;
     unsigned int t;
-
-    *matching_rect = *rect;
-    stack_inspect = new_pnt_stack(DEFAULT_STACK_SIZE_FOR_SEARCHES,
-                                  sizeof qnb);
+    
     scanned_node = &db->root;
     FindInRectIntCBContext context = {
         .db = db,
         .cb = cb,
+        .cluster_cb = cluster_cb,
         .context_cb = context_cb,
         .position = &rect_center,
         .limit = limit
@@ -855,6 +896,9 @@ int find_in_rect(const PanDB * const db,
     for (;;) {
         assert(scanned_node->type == NODE_TYPE_QUAD_NODE);
         get_qrects_from_qbounds(scanned_children_qbounds, &scanned_qbounds);
+        if (cluster != 0) {
+            max_nb_slots_without_clustering = context.limit / (SubSlots) 4U;
+        }
         t = 0U;
         do {
             scanned_node_child = scanned_node->nodes[t];
@@ -863,6 +907,23 @@ int find_in_rect(const PanDB * const db,
                                       matching_rect) == 0) {
                 continue;
             }
+            if (scanned_node_child->bare_node.type == NODE_TYPE_QUAD_NODE &&
+                cluster != 0 &&
+                scanned_node_child->quad_node.sub_slots >
+                max_nb_slots_without_clustering &&
+                fabsf(scanned_child_qbound->edge1.latitude -
+                      scanned_child_qbound->edge0.latitude) < epsilon &&
+                fabsf(scanned_child_qbound->edge1.longitude -
+                      scanned_child_qbound->edge0.longitude) < epsilon) {
+                context.rect = scanned_child_qbound;
+                const int ret = find_in_rect_cluster_context_cb
+                    (&context,
+                        (void *) scanned_node_child, sizeof *scanned_node_child);
+                if (ret != 0) {
+                    return ret;
+                }
+                continue;
+            }
             if (scanned_node_child->bare_node.type == NODE_TYPE_BUCKET_NODE) {
                 const Bucket *bucket = &scanned_node_child->bucket_node.bucket;
                 context.rect = rect;
@@ -885,12 +946,67 @@ int find_in_rect(const PanDB * const db,
         scanned_node = sqnb->quad_node;
         scanned_qbounds = sqnb->qrect;
     }
-    free_pnt_stack(stack_inspect);
-
     return 0;
 }
 
-int init_pan_db(PanDB * const db)
+int find_in_rect(const PanDB * const db,
+                 FindInRectCB cb, FindInRectClusterCB cluster_cb,
+                 void * const context_cb,
+                 const Rectangle2D * const rect,
+                 const SubSlots limit, const Dimension epsilon)
+{
+    if (limit <= (SubSlots) 0) {
+        return 0;
+    }
+    Rectangle2D matching_rects[4];
+    Rectangle2D *matching_rect = &matching_rects[0];
+    PntStack *stack_inspect;
+    unsigned int nb_zones;
+    
+    if (db->layer_type == LAYER_TYPE_FLAT) {
+        matching_rects[0] = *rect;
+        nb_zones = 1U;
+    } else {
+        Rectangle2D orect = *rect;
+        if (orect.edge0.latitude > orect.edge1.latitude) {
+            orect.edge1.latitude +=
+                db->qbounds.edge1.latitude - db->qbounds.edge0.latitude;
+        }
+        if (orect.edge0.longitude > orect.edge1.longitude) {
+            orect.edge1.longitude +=
+                db->qbounds.edge1.longitude - db->qbounds.edge0.longitude;
+        }        
+        nb_zones = find_zones(db, &orect, matching_rects);
+    }
+    assert(nb_zones >= 1U);
+    assert(nb_zones <= 4U);
+    
+    stack_inspect = new_pnt_stack(DEFAULT_STACK_SIZE_FOR_SEARCHES,
+                                  sizeof(QuadNodeWithBounds));
+    if (stack_inspect == NULL) {
+        return -1;
+    }    
+    assert(matching_rect == &matching_rects[0]);
+    int ret;
+    do {
+        ret = find_in_rect_in_zone(matching_rect,
+                                   stack_inspect,
+                                   db,
+                                   cb,
+                                   cluster_cb,
+                                   context_cb,
+                                   matching_rect,
+                                   limit,
+                                   epsilon);
+        matching_rect++;
+    } while (ret == 0 && --nb_zones > 0U);    
+    free_pnt_stack(stack_inspect);
+
+    return ret;
+}
+
+int init_pan_db(PanDB * const db,
+                struct HttpHandlerContext_ * const context)
 {
     pthread_rwlock_init(&db->rwlock_db, NULL);
     init_quad_node(&db->root);
@@ -902,7 +1018,10 @@ int init_pan_db(PanDB * const db)
         app_context.dimension_accuracy;
     db->layer_type = app_context.default_layer_type;
     db->accuracy = app_context.default_accuracy;
+    assert(context != NULL);
+    db->context = context;
     RB_INIT(&db->key_nodes);
+    RB_INIT(&db->expirables);    
     
     return 0;
 }
@@ -928,7 +1047,7 @@ void free_pan_db(PanDB * const db)
                                 scanned_key_node);
         RB_REMOVE(KeyNodes_, &db->key_nodes, scanned_key_node);
         scanned_key_node->slot = NULL;
-        free_key_node(scanned_key_node);
+        free_key_node(db, scanned_key_node);
     }
     stack_inspect = new_pnt_stack((size_t) 8U, sizeof qn);
     stack_quad_nodes_to_delete = new_pnt_stack((size_t) 8U, sizeof qn);
@@ -962,6 +1081,8 @@ void free_pan_db(PanDB * const db)
         free_quad_node(qn);
     }
     free_pnt_stack(stack_quad_nodes_to_delete);
+    assert(db->context != NULL);
+    db->context = NULL;
     pthread_rwlock_destroy(&db->rwlock_db);
 }
 
